@@ -1,4 +1,4 @@
-import re
+import re, sys
 import logging
 from fractions import Fraction as frac
 from collections import defaultdict, ChainMap # for merging dictionaries
@@ -9,8 +9,8 @@ import numpy as np
 
 from .bs4_measures import MeasureList
 from .logger import function_logger, LoggedClass
-from .utils import color2rgba, color_params2rgba, column_order, fifths2name, FORM_DETECTION_REGEX, ordinal_suffix, pretty_dict,\
-    resolve_dir, rgba2attrs, rgba2params, sort_note_list
+from .utils import adjacency_groups, color2rgba, color_params2rgba, column_order, fifths2name, FORM_DETECTION_REGEX, \
+    get_quarterbeats_length, ordinal_suffix, pretty_dict, resolve_dir, rgba2attrs, rgba2params, sort_note_list
 
 
 
@@ -61,7 +61,11 @@ class _MSCX_bs4(LoggedClass):
         self.read_only = read_only
         self.first_mc = 1
         self.measure_nodes = {}
-        self.tags = {} # only used if not self.read_only
+        """{staff -> {MC -> tag} }"""
+
+        self.tags = {}  # only used if not self.read_only
+        """{MC -> {staff -> {voice -> tag} } }"""
+
         self.has_annotations = False
         self.n_form_labels = 0
         self._ml = None
@@ -226,7 +230,7 @@ Use 'ms3 convert' command or pass parameter 'ms' to Score to temporally convert.
         try:
             mscx_string = bs4_to_mscx(self.soup)
         except:
-            logging.error(f"BeautifulSoup object is None.")
+            self.logger.error(f"Couldn't output MSCX because of the following error:\n{sys.exc_info()[1]}")
             return False
         with open(resolve_dir(filepath), 'w', encoding='utf-8') as file:
             file.write(mscx_string)
@@ -656,13 +660,40 @@ The first ending MC {mc} is being used. Suppress this warning by using disambigu
         assert self.soup is not None, "The file's XML needs to be loaded. Get metadata from the 'metadata' property or use the method make_writeable()"
         nav_str2str = lambda s: '' if s is None else str(s)
         data = {tag['name']: nav_str2str(tag.string) for tag in self.soup.find_all('metaTag')}
+        if 'reviewer' in data:
+            if 'reviewers' in data:
+                logger.warning("Score properties contain a superfluous key called 'reviewer'. "
+                               "Please merge with the value for 'reviewers' and delete.")
+            else:
+                logger.info("The key 'reviewer' contained in the Score properties was automatically "
+                            "renamed to 'reviewers' when extracting metadata.")
+                data['reviewers'] = data['reviewer']
+                del(data['reviewer'])
+        if 'annotator' in data:
+            if 'annotators' in data:
+                logger.warning("Score properties contain a superfluous key called 'annotator'. "
+                               "Please merge with the value for 'annotators' and delete.")
+            else:
+                logger.info("The key 'annotator' contained in the Score properties was automatically "
+                            "renamed to 'annotators' when extracting metadata.")
+                data['annotators'] = data['annotator']
+                del(data['annotator'])
         data['musescore'] = self.version
         last_measure = self.ml.iloc[-1]
         data['last_mc'] = int(last_measure.mc)
         data['last_mn'] = int(last_measure.mn)
+        lqb, lqbu = get_quarterbeats_length(self.ml)
+        data['length_qb'] = lqb
+        data['length_qb_unfolded'] = lqbu
         data['label_count'] = len(self.get_raw_labels())
-        data['TimeSig'] = dict(self.ml.loc[self.ml.timesig != self.ml.timesig.shift(), ['mc', 'timesig']].itertuples(index=False, name=None))
-        data['KeySig']  = dict(self.ml.loc[self.ml.keysig != self.ml.keysig.shift(), ['mc', 'keysig']].itertuples(index=False, name=None))
+        ts_groups, _ = adjacency_groups(self.ml.timesig)
+        mc_ts = self.ml.groupby(ts_groups)[['mc', 'timesig']].head(1)
+        timesigs = dict(mc_ts.values)
+        data['TimeSig'] = timesigs
+        ks_groups, _ = adjacency_groups(self.ml.keysig)
+        mc_ks = self.ml.groupby(ks_groups)[['mc', 'keysig']].head(1)
+        keysigs = dict(mc_ks.values)
+        data['KeySig']  = keysigs
         annotated_key = None
         for harmony_tag in self.soup.find_all('Harmony'):
             label = harmony_tag.find('name')
@@ -674,7 +705,13 @@ The first ending MC {mc} is being used. Suppress this warning by using disambigu
         if annotated_key is not None:
             data['annotated_key'] = annotated_key
         if len(self.nl.index) == 0:
+            data['all_notes_qb'] = 0.
+            data['n_onsets'] = 0
             return data
+        data['all_notes_qb'] = round((self.nl.duration * 4.).sum(), 2)
+        not_tied = ~self.nl.tied.isin((0, -1))
+        data['n_onsets'] = sum(not_tied)
+        data['n_onset_positions'] = self.nl[not_tied].groupby(['mc', 'mc_onset']).size().shape[0]
         staff_groups = self.nl.groupby('staff').midi
         ambitus = {t.staff: {'min_midi': t.midi, 'min_name': fifths2name(t.tpc, t.midi)}
                         for t in self.nl.loc[staff_groups.idxmin(), ['staff', 'tpc', 'midi', ]].itertuples(index=False)}
@@ -917,6 +954,9 @@ but the keys of _MSCX_bs4.tags[{mc}][{staff}] are {dict_keys}."""
         -------
 
         """
+        if pd.isnull(label) and len(kwargs) == 0:
+            self.logger.error(f"Label cannot be '{label}'")
+            return False
         assert mc_onset >= 0, f"Cannot attach label {label} to negative onset {mc_onset} at MC {mc}, staff {staff}, voice {voice}"
         self.make_writeable()
         if mc not in self.tags:
@@ -1648,7 +1688,7 @@ def format_node(node, indent):
     node_name = node.name
     # The following tags are exceptionally not abbreviated when empty,
     # so for instance you get <metaTag></metaTag> and not <metaTag/>
-    if node_name in ['continueAt', 'continueText', 'endText', 'text', 'LayerTag', 'metaTag', 'trackName']:
+    if node_name in ['continueAt', 'continueText', 'endText', 'text', 'LayerTag', 'programRevision', 'metaTag', 'trackName']:
         return f"{space}{make_oneliner(node)}\n"
     children = node.find_all(recursive=False)
     if len(children) > 0:
