@@ -7,9 +7,10 @@ from collections import defaultdict
 import pandas as pd
 import numpy as np
 
-from .utils import abs2rel_key, changes2list, DCML_REGEX, fifths2iv, fifths2name, fifths2pc, fifths2rn, fifths2sd, \
-    labels2global_tonic, map2elements, name2fifths, rel2abs_key, resolve_relative_keys, roman_numeral2fifths, \
-    series_is_minor, split_alternatives, split_scale_degree, str_is_minor, transform, sort_tpcs
+from .utils import abs2rel_key, changes2list, DCML_REGEX, features2tpcs, fifths2iv, fifths2name, fifths2pc, fifths2rn, fifths2sd, \
+    map2elements, name2fifths, rel2abs_key, resolve_relative_keys, roman_numeral2fifths, \
+    series_is_minor, split_alternatives, transform
+from .transformations import compute_chord_tones, labels2global_tonic
 from .logger import function_logger
 
 
@@ -560,362 +561,6 @@ def propagate_pedal(df, relative=True, drop_pedalend=True, cols={}):
     return df
 
 
-@function_logger
-def compute_chord_tones(df, bass_only=False, expand=False, cols={}):
-    """
-    Compute the chord tones for DCML harmony labels. They are returned as lists
-    of tonal pitch classes in close position, starting with the bass note. The
-    tonal pitch classes represent intervals relative to the local tonic:
-
-    -2: Second below tonic
-    -1: fifth below tonic
-    0: tonic
-    1: fifth above tonic
-    2: second above tonic, etc.
-
-    The labels need to have undergone :py:func:`split_labels` and :py:func:`propagate_keys`.
-    Pedal points are not taken into account.
-
-    Uses: :py:func:`features2tpcs`
-
-    Parameters
-    ----------
-    df : :obj:`pandas.DataFrame`
-        Dataframe containing DCML chord labels that have been split by split_labels()
-        and where the keys have been propagated using propagate_keys(add_bool=True).
-    bass_only : :obj:`bool`, optional
-        Pass True if you need only the bass note.
-    expand : :obj:`bool`, optional
-        Pass True if you need chord tones and added tones in separate columns.
-    cols : :obj:`dict`, optional
-        In case the column names for ``['mc', 'numeral', 'form', 'figbass', 'changes', 'relativeroot', 'localkey', 'globalkey']`` deviate, pass a dict, such as
-
-        .. code-block:: python
-
-            {'mc':              'mc',
-             'numeral':         'numeral_col_name',
-             'form':            'form_col_name',
-             'figbass':         'figbass_col_name',
-             'changes':         'changes_col_name',
-             'relativeroot':    'relativeroot_col_name',
-             'localkey':        'localkey_col_name',
-             'globalkey':       'globalkey_col_name'}
-
-        You may also deactivate columns by setting them to None, e.g. {'changes': None}
-
-    Returns
-    -------
-    :obj:`pandas.Series` or :obj:`pandas.DataFrame`
-        For every row of `df` one tuple with chord tones, expressed as tonal pitch classes.
-        If `expand` is True, the function returns a DataFrame with four columns:
-        Two with tuples for chord tones and added tones, one with the chord root,
-        and one with the bass note.
-    """
-
-    df = df.copy()
-    ### If the index is not unique, it has to be temporarily replaced
-    tmp_index = not df.index.is_unique
-    if tmp_index:
-        ix = df.index
-        df.reset_index(drop=True, inplace=True)
-
-    features = ['mc', 'numeral', 'form', 'figbass', 'changes', 'relativeroot', 'localkey', 'globalkey']
-    for col in features:
-        if col in df.columns and not col in cols:
-            cols[col] = col
-    local_minor, global_minor = f"{cols['localkey']}_is_minor", f"{cols['globalkey']}_is_minor"
-    if not local_minor in df.columns:
-        df[local_minor] = series_is_minor(df[cols['localkey']], is_name=False)
-        logger.debug(f"Boolean column '{local_minor}' created.'")
-    if not global_minor in df.columns:
-        df[global_minor] = series_is_minor(df[cols['globalkey']], is_name=True)
-        logger.debug(f"Boolean column '{global_minor}' created.'")
-
-    param_cols = {col: cols[col] for col in ['numeral', 'form', 'figbass', 'changes', 'relativeroot', 'mc'] if cols[col] is not None}
-    param_cols['minor'] = local_minor
-    param_tuples = list(df[param_cols.values()].itertuples(index=False, name=None))
-    #result_dict = {t: features2tpcs(**{a:b for a, b in zip(param_cols.keys(), t)}, bass_only=bass_only, merge_tones=not expand, logger=logger) for t in set(param_tuples)}
-    result_dict = {}
-    if bass_only:
-        default = None
-    elif not expand:
-        default = tuple()
-    else:
-        default = {
-            'chord_tones': tuple(),
-            'added_tones': tuple(),
-            'root': None,
-        }
-    for t in set(param_tuples):
-        try:
-            result_dict[t] = features2tpcs(**{a: b for a, b in zip(param_cols.keys(), t)}, bass_only=bass_only,
-                                           merge_tones=not expand, logger=logger)
-        except:
-            result_dict[t] = default
-            logger.warning(str(sys.exc_info()[1]))
-    if expand:
-        res = pd.DataFrame([result_dict[t] for t in param_tuples], index=df.index)
-        res['bass_note'] = res.chord_tones.apply(lambda l: np.nan if pd.isnull(l) or len(l) == 0 else l[0])
-        res[['root', 'bass_note']] = res[['root', 'bass_note']].astype('Int64')
-    else:
-        res = pd.Series([result_dict[t] for t in param_tuples], index=df.index)
-
-    if tmp_index:
-        res.index = ix
-
-    return res
-
-
-@function_logger
-def features2tpcs(numeral, form=None, figbass=None, changes=None, relativeroot=None, key='C', minor=None,
-                  merge_tones=True, bass_only=False, mc=None):
-    """
-    Given the features of a chord label, this function returns the chord tones
-    in the order of the inversion, starting from the bass note. The tones are
-    expressed as tonal pitch classes, where -1=F, 0=C, 1=G etc.
-
-    Uses: :py:func:`~.utils.changes2list`, :py:func:`~.utils.name2fifths`, :py:func:`~.utils.resolve_relative_keys`, :py:func:`~.utils.roman_numeral2fifths`,
-    :py:func:`~.utils.sort_tpcs`, :py:func:`~.utils.str_is_minor`
-
-    Parameters
-    ----------
-    numeral: :obj:`str`
-        Roman numeral of the chord's root
-    form: {None, 'M', 'o', '+' '%'}, optional
-        Indicates the chord type if not a major or minor triad (for which ``form`` is None).
-        '%' and 'M' can only occur as tetrads, not as triads.
-    figbass: {None, '6', '64', '7', '65', '43', '2'}, optional
-        Indicates chord's inversion. Pass None for triad root position.
-    changes: :obj:`str`, optional
-        Added steps such as '+6' or suspensions such as '4' or any combination such as (9+64).
-        Numbers need to be in descending order.
-    relativeroot: :obj:`str`, optional
-        Pass a Roman scale degree if `numeral` is to be applied to a different scale
-        degree of the local key, as in 'V65/V'
-    key : :obj:`str` or :obj:`int`, optional
-        The local key expressed as the root's note name or a tonal pitch class.
-        If it is a name and `minor` is `None`, uppercase means major and lowercase minor.
-        If it is a tonal pitch class, `minor` needs to be specified.
-    minor : :obj:`bool`, optional
-        Pass True for minor and False for major. Can be omitted if `key` is a note name.
-        This affects calculation of chords related to III, VI and VII.
-    merge_tones : :obj:`bool`, optional
-        Pass False if you want the function to return two tuples, one with (potentially suspended)
-        chord tones and one with added notes.
-    bass_only : :obj:`bool`, optional
-        Return only the bass note instead of all chord tones.
-    mc : int or str
-        Pass measure count to display it in warnings.
-
-    """
-    if pd.isnull(numeral) or numeral == '@none':
-        if bass_only or merge_tones:
-            return np.nan
-        else:
-            return {
-                'chord_tones': np.nan,
-                'added_tones': np.nan,
-                'root': np.nan,
-            }
-    form, figbass, changes, relativeroot = tuple(
-        '' if pd.isnull(val) else val for val in (form, figbass, changes, relativeroot))
-    label = f"{numeral}{form}{figbass}{'(' + changes + ')' if changes != '' else ''}{'/' + relativeroot if relativeroot != '' else ''}"
-    MC = '' if mc is None else f'MC {mc}: '
-    if minor is None:
-        try:
-            minor = str_is_minor(key, is_name=True)
-            logger.debug(f"Mode inferred from {key}.")
-        except:
-            raise ValueError(f"If parameter 'minor' is not specified, 'key' needs to be a string, not {key}")
-
-    key = name2fifths(key)
-
-    if form in ['%', 'M', '+M']:
-        assert figbass in ['7', '65', '43',
-                           '2'], f"{MC}{label}: {form} requires figbass (7, 65, 43, or 2) since it specifies a chord's seventh."
-
-    if relativeroot != '':
-        resolved = resolve_relative_keys(relativeroot, minor)
-        rel_minor = str_is_minor(resolved, is_name=False)
-        transp = roman_numeral2fifths(resolved, minor)
-        logger.debug(
-            f"{MC}Chord applied to {relativeroot}. Therefore transposing it by {transp} fifths.")
-        return features2tpcs(numeral=numeral, form=form, figbass=figbass, relativeroot=None, changes=changes,
-                             key=key + transp, minor=rel_minor, merge_tones=merge_tones, bass_only=bass_only, mc=mc,
-                             logger=logger)
-
-    if numeral.lower() == '#vii' and not minor:
-        # logger.warning(
-        #     f"{MC}{label} in a major context is most probably an annotation error.")
-        logger.warning(f"{MC}{numeral} in major context corrected to {numeral[1:]}.")
-        numeral = numeral[1:]
-
-    root_alteration, num_degree = split_scale_degree(numeral, count=True, logger=logger)
-
-    # build 2-octave diatonic scale on C major/minor
-    root = ['I', 'II', 'III', 'IV', 'V', 'VI', 'VII'].index(num_degree.upper())
-    tpcs = 2 * [i + key for i in (0, 2, -3, -1, 1, -4, -2)] if minor else 2 * [i + key for i in (0, 2, 4, -1, 1, 3, 5)]
-    # starting the scale from chord root, i.e. root will be tpcs[0], the chord's seventh tpcs[6] etc.
-    tpcs = tpcs[root:] + tpcs[:root]
-    root = tpcs[0] + 7 * root_alteration
-    tpcs[0] = root  # octave stays diatonic, is not altered
-    logger.debug(f"{num_degree}: The {'minor' if minor else 'major'} scale starting from the root: {tpcs}")
-
-    def set_iv(chord_interval, interval_size):
-        """ Add to the interval of a given chord interval in `tpcs` (both viewed from the root note).
-
-        Parameters
-        ----------
-        chord_interval : :obj:`int`
-            Pass 0 for the root note, 2 for the third, 8 for the ninth etc.
-        interval_size : :obj:`int`
-            Stack-of-fifths interval.
-        """
-        nonlocal tpcs, root
-        iv = root + interval_size
-        tpcs[chord_interval] = iv
-        tpcs[chord_interval + 7] = iv
-
-    is_triad = figbass in ['', '6', '64']
-    is_seventh_chord = figbass in ['7', '65', '43', '2']
-    if not is_triad and not is_seventh_chord:
-        raise ValueError(f"{MC}{figbass} is not a valid chord inversion.")
-
-    if form == 'o':
-        set_iv(2, -3)
-        set_iv(4, -6)
-        if is_seventh_chord:
-            set_iv(6, -9)
-    elif form == '%':
-        set_iv(2, -3)
-        set_iv(4, -6)
-        set_iv(6, -2)
-    elif form == '+':
-        set_iv(2, 4)
-        set_iv(4, 8)
-        if is_seventh_chord:
-            set_iv(6, -2)
-    elif form == '+M':
-        set_iv(2, 4)
-        set_iv(4, 8)
-        set_iv(6, 5)
-    else:  # triad with or without major or minor seven
-        set_iv(4, 1)
-        if num_degree.isupper():
-            set_iv(2, 4)
-        else:
-            set_iv(2, -3)
-        if form == 'M':
-            set_iv(6, 5)
-        elif is_seventh_chord:
-            set_iv(6, -2)
-
-    tone_functions = (0, 2, 4, 6) if is_seventh_chord else (0, 2, 4)
-    root_position = {i: [tpcs[i]] for i in tone_functions}
-    replacements  = {i: [] for i in tone_functions}
-
-    def replace_chord_tone(which, by):
-        nonlocal root_position, replacements
-        if which in root_position:
-            root_position[which] = []
-            replacements[which].insert(0, by)
-        else:
-            logger.warning(f"Only chord tones [0,2,4,(6)] can be replaced, not {which}")
-
-    # apply changes
-    alts = changes2list(changes, sort=False)
-    added_notes = []
-    for full, add_remove, acc, chord_interval in alts:
-        added = add_remove == '+'
-        substracted = add_remove == '-'
-        replacing_upper = add_remove == '^'
-        replacing_lower = add_remove == 'v'
-        chord_interval = int(chord_interval) - 1
-        ### From here on, `chord_interval` is decremented, i.e. the root is 0, the seventh is 6 etc. (just like in `tpcs`)
-        if (chord_interval == 0 and not substracted) or chord_interval > 13:
-            logger.warning(
-                f"{MC}Change {full} is meaningless and ignored because it concerns chord tone {chord_interval + 1}.")
-            continue
-        next_octave = chord_interval > 7
-        shift = 7 * (acc.count('#') - acc.count('b'))
-        new_val = tpcs[chord_interval] + shift
-        if substracted:
-            if chord_interval not in tone_functions:
-                logger.warning(
-                    f"{MC}The change {full} has no effect because it concerns an interval which is not implied by {numeral}{form}{figbass}.")
-            else:
-                root_position[chord_interval] = []
-        elif added:
-            added_notes.append(new_val)
-        elif next_octave:
-            if any((replacing_lower, replacing_upper, substracted)):
-                logger.warning(f"{MC}{full[0]} has no effect in {full}  because the interval is larger than an octave.")
-            added_notes.append(new_val)
-        elif chord_interval in [1, 3, 5]:  # these are changes to scale degree 2, 4, 6 that replace the lower neighbour unless they have a # or ^
-            if '#' in acc or replacing_upper:
-                if '#' in acc and replacing_upper:
-                    logger.warning(f"{MC}^ is redundant in {full}.")
-                if chord_interval == 5 and is_triad:  # leading tone to 7 but not in seventh chord
-                    added_notes.append(new_val)
-                else:
-                    replace_chord_tone(chord_interval + 1, new_val)
-            else:
-                if replacing_lower:
-                    logger.warning(f"{MC}v is redundant in {full}.")
-                replace_chord_tone(chord_interval - 1, new_val)
-        else:  # chord tone alterations
-            if replacing_lower:
-                # TODO: This must be possible, e.g. V(6v5) where 5 is suspension of 4
-                logger.warning(f"{MC}{full} -> chord tones cannot replace neighbours, use + instead.")
-            elif chord_interval == 6 and figbass != '7':  # 7th are a special case:
-                if figbass == '':  # in root position triads they are added
-                    # TODO: The standard is lacking a distinction, because the root in root pos. can also be replaced from below!
-                    added_notes.append(new_val)
-                elif figbass in ['6', '64'] or '#' in acc:  # in inverted triads they replace the root, as does #7
-                    replace_chord_tone(0, new_val)
-                else:  # otherwise they are unclear
-                    logger.warning(
-                        f"{MC}In seventh chords, such as {label}, it is not clear whether the {full} alters the 7 or replaces the 8 and should not be used.")
-            elif tpcs[chord_interval] == new_val:
-                logger.warning(
-                    f"{MC}The change {full} has no effect in {numeral}{form}{figbass}")
-            else:
-                root_position[chord_interval] = [new_val]
-
-    figbass2bass = {
-        '': 0,
-        '7': 0,
-        '6': 1,
-        '65': 1,
-        '64': 2,
-        '43': 2,
-        '2': 3
-    }
-    bass = figbass2bass[figbass]
-    chord_tones = []
-    for tf in tone_functions[bass:] + tone_functions[:bass]:
-        chord_tone, replacing_tones = root_position[tf], replacements[tf]
-        if chord_tone == replacing_tones == []:
-            logger.debug(f"{MC}{label} results in a chord without {tf + 1}.")
-        if chord_tone != []:
-            chord_tones.append(chord_tone[0])
-            if replacing_tones != []:
-                logger.warning(f"{MC}{label} results in a chord tone {tf + 1} AND its replacement(s) {replacing_tones}.")
-        chord_tones.extend(replacing_tones)
-
-    bass_tpc = chord_tones[0]
-    if bass_only:
-        return bass_tpc
-    elif merge_tones:
-        return tuple(sort_tpcs(chord_tones + added_notes, start=bass_tpc))
-    else:
-        return {
-            'chord_tones': tuple(chord_tones),
-            'added_tones': tuple(added_notes),
-            'root': root,
-        }
-
-
 ########################################################################################################################
 #                                           MOMENTARILY NOT IN USE:
 ########################################################################################################################
