@@ -1,22 +1,117 @@
 """Functions for transforming DataFrames as output by ms3."""
 import sys
+from collections.abc import Iterable
 from fractions import Fraction as frac
 from functools import reduce
 
 import numpy as np
 import pandas as pd
 
-
 from .logger import function_logger
-from .utils import features2tpcs, make_interval_index, rel2abs_key, resolve_relative_keys, roman_numeral2fifths, \
-    roman_numeral2semitones, series_is_minor, transform, transpose_changes
+from .utils import adjacency_groups, features2tpcs, fifths2name, fifths2pc, interval_overlap, interval_overlap_size, make_interval_index,\
+    make_continuous_offset, make_playthrough2mc, name2fifths, nan_eq, rel2abs_key,\
+    replace_index_by_intervals, resolve_relative_keys, roman_numeral2fifths, \
+    roman_numeral2semitones, series_is_minor, transform, transpose_changes, unfold_repeats
 
 
 def add_localkey_change_column(at, key_column='localkey'):
-    key_segment = at[key_column] != at[key_column].shift()
+    key_segment, _ = adjacency_groups(at[key_column], na_values='ffill')
     return pd.concat([at, key_segment.rename('key_segment')], axis=1)
 
 
+@function_logger
+def add_quarterbeats_col(df, offset_dict, interval_index=False):
+    """ Insert a column measuring the distance of events from MC 1 in quarter notes. If no 'mc_onset' column is present,
+        the column corresponds to the ``insert_after`` column's measure counts.
+
+    Parameters
+    ----------
+    df : :obj:`pandas.DataFrame`
+        DataFrame with an ``mc_playthrough`` and an ``mc_onset`` column.
+    offset_dict : :obj:`pandas.Series` or :obj:`dict`
+        | If unfolded: {mc_playthrough -> offset}
+        | Otherwise: {mc -> offset}
+        | You can create the dict using the function :py:meth:`Parse.get_continuous_offsets()<ms3.parse.Parse.get_continuous_offsets>`
+    interval_index : :obj:`bool`, optional
+        Defaults to False. Pass True to replace the index with an :obj:`pandas.IntervalIndex` (depends on the successful
+        creation of the column ``duration_qb``).
+
+    Returns
+    -------
+
+    """
+    if offset_dict is None:
+        logger.warning(f"No offset_dict was passed: Not adding quarterbeats.")
+        return df
+    if 'quarterbeats' not in df.columns:
+        if 'mc_playthrough' in df.columns:
+            insert_before = 'mc_playthrough'
+        elif 'mc' in df.columns:
+            insert_before = 'mc'
+        else:
+            logger.error("Expected to have at least one column called 'mc' or 'mc_playthrough'.")
+            return df
+        df = df.copy()
+        quarterbeats = df[insert_before].map(offset_dict)
+        if 'mc_onset' in df.columns:
+            quarterbeats += df.mc_onset * 4
+        insert_here = df.columns.get_loc(insert_before)
+        df.insert(insert_here, 'quarterbeats', quarterbeats)
+        if 'duration_qb' not in df.columns:
+            if 'duration' in df.columns:
+                dur = (df.duration * 4).astype(float).round(3)
+                df.insert(insert_here + 1, 'duration_qb', dur)
+            elif 'end' in offset_dict:
+                present_qb = df.quarterbeats.notna()
+                try:
+                    ivs = make_interval_index(df.loc[present_qb, 'quarterbeats'].astype(float).round(3),
+                                              end_value=float(offset_dict['end']), logger=logger)
+                    df.insert(insert_here + 1, 'duration_qb', pd.NA)
+                    df.loc[present_qb, 'duration_qb'] = ivs.length
+                except Exception:
+                    logger.warning(
+                        "Error while creating durations from quarterbeats column. Check consistency (quarterbeats need to be monotically ascending; 'end' value in offset_dict needs to be larger than the last quarterbeat).")
+            else:
+                logger.warning("Column 'duration_qb' could not be created.")
+    else:
+        logger.debug("quarterbeats column was already present.")
+    if interval_index and all(c in df.columns for c in ('quarterbeats', 'duration_qb')):
+        df = replace_index_by_intervals(df)
+    return df
+
+
+def add_weighted_grace_durations(notes, weight=1/2):
+    """ For a given notes table, change the 'duration' value of all grace notes, weighting it by ``weight``.
+
+    Parameters
+    ----------
+    notes : :obj:`pandas.DataFrame`
+        Notes table containing the columns 'duration', 'nominal_duration', 'scalar'
+    weight : :obj:`Fraction` or :obj:`float`
+        Value by which to weight duration of all grace notes. Defaults to a half.
+
+    Returns
+    -------
+    :obj:`pandas.DataFrame`
+        Copy of ``notes`` with altered duration values.
+    """
+    if 'gracenote' not in notes.columns:
+        return notes
+    grace = notes.gracenote.notna()
+    if grace.sum() == 0:
+        return notes
+    notes = notes.copy()
+    assert all(col in notes.columns for col in ('duration', 'nominal_duration', 'scalar')),\
+        "Needs to contain columns 'duration', 'nominal_duration', 'scalar'"
+    gracenotes = notes[grace]
+    new_durations = gracenotes.scalar * gracenotes.nominal_duration * frac(weight)
+    notes.loc[grace, 'duration'] = new_durations
+    if 'duration_qb' in notes.columns:
+        notes.loc[grace, 'duration_qb'] = (new_durations * 4).astype(float)
+        if isinstance(notes.index, pd.IntervalIndex):
+            notes = replace_index_by_intervals(notes)
+    return notes
+
 
 @function_logger
 def compute_chord_tones(df, bass_only=False, expand=False, cols={}):
@@ -125,6 +220,54 @@ def compute_chord_tones(df, bass_only=False, expand=False, cols={}):
     return res
 
 
+@function_logger
+def dfs2quarterbeats(dfs, measures, unfold=False, quarterbeats=True, interval_index=True):
+    """ Pass one or several DataFrames and one measures table to unfold repeats and/or add quarterbeats columns and/or index.
+
+    Parameters
+    ----------
+    dfs : (:obj:`list` of) :obj:`pandas.DataFrame`
+        DataFrame(s) that are to be unfolded and/or receive quarterbeats.
+    measures : :obj:`pandas.DataFrame`
+    unfold : :obj:`bool`, optional
+    quarterbeats : :obj:`bool`, optional
+    interval_index : :obj:`bool`, optional
+
+    Returns
+    -------
+    :obj:`list` of :obj:`pandas.DataFrame`
+        Altered copies of `dfs`.
+    """
+    assert sum((unfold, quarterbeats, interval_index)) >= 1, "At least one of the 'unfold', 'quarterbeats', and 'interval_index' arguments needs to be True."
+    if isinstance(dfs, pd.DataFrame):
+        dfs = [dfs]
+    if interval_index:
+        quarterbeats = True
+    if unfold:
+        playthrough2mc = make_playthrough2mc(measures, logger=logger)
+        if len(playthrough2mc) == 0:
+            logger.warning("Added quarterbeats without unfolding because of incorrect repeat structure.")
+            unfold = False
+    if unfold:
+        dfs = [unfold_repeats(df, playthrough2mc, logger=logger) if df is not None else df for df in dfs]
+        if quarterbeats:
+            unfolded_measures = unfold_repeats(measures, playthrough2mc, logger=logger)
+            continuous_offset = make_continuous_offset(unfolded_measures, logger=logger)
+            dfs = [add_quarterbeats_col(df, continuous_offset, interval_index=interval_index, logger=logger)
+                   if df is not None else df for df in dfs]
+    elif quarterbeats:
+        if 'volta' in measures.columns:
+            logger.debug(f"No quarterbeats are assigned to first endings. Pass unfold=True to "
+                         f"compute quarterbeats for a full playthrough.")
+            if 3 in measures.volta.values:
+                logger.debug(
+                    f"Piece contains third endings, note that only second endings are taken into account.")
+            measures = measures.drop(index=measures[measures.volta.fillna(2) != 2].index, columns='volta')
+        continuous_offset = make_continuous_offset(measures, logger=logger)
+        dfs = [add_quarterbeats_col(df, continuous_offset, interval_index=interval_index, logger=logger)
+               if df is not None else df for df in dfs]
+    return dfs
+
 
 @function_logger
 def get_chord_sequences(at, major_minor=True, level=None, column='chord'):
@@ -160,7 +303,6 @@ def get_chord_sequences(at, major_minor=True, level=None, column='chord'):
                 at = transform_multiple(at, 'key_segment', level=level)
             else:
                 at = add_localkey_change_column(at)
-                at.key_segment = at.key_segment.cumsum()
         res = {}
         for i, df in at.groupby('key_segment'):
             row = df.iloc[0]
@@ -180,12 +322,13 @@ def get_chord_sequences(at, major_minor=True, level=None, column='chord'):
         return sequences
 
 
-
-def group_annotations_by_features(at, features='numeral', dropna=True):
+@function_logger
+def group_annotations_by_features(at, features='numeral'):
     """ Drop exact repetitions of one or several feature columns when occurring under the same localkey (and pedal point).
     For example, pass ``features = ['numeral', 'form', 'figbass']`` to drop rows where all three features are identical
     with the previous row _and_ the localkey stays the same. If the column ``duration_qb`` is present, it is updated
     with the new durations, as would be the IntervalIndex if there is one.
+    Uses: nan_eq()
 
     Parameters
     ----------
@@ -237,14 +380,14 @@ def group_annotations_by_features(at, features='numeral', dropna=True):
     if isinstance(features, str):
         features = [features]
     qb_cols = ['quarterbeats', 'duration_qb']
-    safety_cols = ['globalkey', 'localkey', 'pedal']
+    safety_cols = ['globalkey', 'localkey', 'pedal'] # if present, these prevent grouping because they change meaning
     keep_cols = ['mc', 'mn', 'mc_onset', 'mn_onset', 'timesig', 'staff', 'voice',
                  'volta', 'label', 'globalkey', 'localkey', 'globalkey_is_minor', 'localkey_is_minor', 'special',
                  'pedal']
     if 'numeral' in features:
         safety_cols.append('relativeroot')
         keep_cols.append('relativeroot')
-    safety = [f for f in safety_cols if f not in features and f in at.columns]
+    safety = [f for f in safety_cols if f not in features and f in at.columns and len(at[f].unique()) > 1]
     cols = []
     for f in features + safety:
         if f in at.columns:
@@ -255,31 +398,20 @@ def group_annotations_by_features(at, features='numeral', dropna=True):
     if len(features) == 0:
         return at
 
-    def column_shift_mask(a, b):
-        """Sets those values of Series a to True where a value in the Serie b is different from its predecessor."""
-        nan_eq = lambda b, b_previous: (b == b_previous).fillna(False) | pd.isnull(b) & pd.isnull(b_previous)
-        return a | ~nan_eq(b, b.shift())
-
-    # The change mask is True for every row where either of the feature or safety columns is different from its previous value.
-    change_mask = reduce(column_shift_mask, (col for _, col in at[cols].iteritems()), pd.Series(False, index=at.index))
-    if dropna:
-        change_mask &= at[features].notna().any(axis=1)
-
-    def sum_durations(df):
-        if len(df) == 1:
-            return df
-        ix = df.index[0]
-        row = df.iloc[[0]]
-        new_duration = df.duration_qb.sum()
-        row.loc[ix, 'duration_qb'] = new_duration
-        if isinstance(ix, pd.Interval) or (isinstance(ix, tuple) and isinstance(ix[-1], pd.Interval)):
-            row.index = pd.IntervalIndex.from_tuples([(ix.left, ix.left + new_duration)], closed=ix.closed)
-        return row
 
     if all(c in at.columns for c in qb_cols):
-        res = at.groupby(change_mask.cumsum(), group_keys=False).apply(sum_durations)
-        keep_cols = ['mc'] + qb_cols + keep_cols[1:]
+        na_values = ['ffill' if c == 'numeral' else 'group' for c in cols]
+        logger.info(f"Grouping with the following na_values settings: {dict(zip(cols, na_values))}")
+        res = segment_by_adjacency_groups(at, cols, na_values=na_values, logger=logger)
+        keep_cols = qb_cols + keep_cols
     else:
+        def column_shift_mask(a, b):
+            """Sets those values of Series a to True where a value in the Series b is different from its predecessor."""
+            return a | ~nan_eq(b, b.shift())
+        # The change mask is True for every row where either of the feature or safety columns is different from its previous value.
+        change_mask = reduce(column_shift_mask,
+                             (col for _, col in at[cols].iteritems()),
+                             pd.Series(False, index=at.index))
         res = at[change_mask]
     keep_cols = [c for c in keep_cols if c in at.columns] + features
     res = res[keep_cols]
@@ -293,7 +425,7 @@ def group_annotations_by_features(at, features='numeral', dropna=True):
     if 'numeral' in res.columns:
         try:
             res = pd.concat([res, compute_chord_tones(res, expand=True)], axis=1)
-        except:
+        except Exception:
             pass
     return res
 
@@ -471,14 +603,14 @@ def make_gantt_data(at, last_mn=None, relativeroots=True, mode_agnostic_adjacenc
         return pd.DataFrame()
 
     at.sort_values(position_col, inplace=True)
-    at.index = make_interval_index(at[position_col], end_value=last_val)
+    at.index = make_interval_index(at[position_col], end_value=last_val, logger=logger)
 
     at['localkey_resolved'] = transform(at, resolve_relative_keys, ['localkey', 'globalkey_is_minor'])
 
     key_groups = at.loc[at.localkey != at.localkey.shift(), [position_col, 'localkey', 'localkey_resolved', 'globalkey',
                                                              'globalkey_is_minor']] \
         .rename(columns={position_col: 'Start'})
-    iix = make_interval_index(key_groups.Start, end_value=last_val)
+    iix = make_interval_index(key_groups.Start, end_value=last_val, logger=logger)
     key_groups.index = iix
     fifths = transform(key_groups, roman_numeral2fifths, ['localkey_resolved', 'globalkey_is_minor']).rename('fifths')
     semitones = transform(key_groups, roman_numeral2semitones, ['localkey_resolved', 'globalkey_is_minor']).rename(
@@ -561,6 +693,96 @@ def make_gantt_data(at, last_mn=None, relativeroots=True, mode_agnostic_adjacenc
                          ('<br>Tonicized local scale degree: ' + res.relativeroot).fillna('')
     return res
 
+@function_logger
+def notes2pcvs(notes,
+               pitch_class_format='tpc',
+               normalize=False,
+               long=False,
+               fillna=True,
+               additional_group_cols=None,
+               ensure_columns=None):
+    """
+
+    Parameters
+    ----------
+    notes : :obj:`pandas.DataFrame`
+        Note table to be transformed into a wide or long table of Pitch Class Vectors by grouping via the first (or only)
+        index level. The DataFrame needs containing at least the columns 'duration_qb' and 'tpc' or 'midi', depending
+        on ``pitch_class_format``.
+    pitch_class_format : :obj:`str`, optional
+        | Defines the type of pitch classes to use for the vectors.
+        | 'tpc' (default): tonal pitch class, such that -1=F, 0=C, 1=G etc.
+        | 'name': tonal pitch class as spelled pitch, e.g. 'C', 'F#', 'Abb' etc.
+        | 'pc': chromatic pitch classes where 0=C, 1=C#/Db, ... 11=B/Cb.
+        | 'midi': original MIDI numbers; the result are pitch vectors, not pitch class vectors.
+    normalize : :obj:`bool`, optional
+        By default, the PCVs contain absolute durations in quarter notes. Pass True to normalize
+        the PCV for each group.
+    long : :obj:`bool`, optional
+        By default, the resulting DataFrames have wide format, i.e. each row contains the PCV
+        for one slice. Pass True if you need long format instead, i.e. with a non-unique
+        :obj:`pandas.IntervalIndex` and two columns, ``[('tpc'|'midi'), 'duration_qb']``
+        where the first column's name depends on ``pitch_class_format``.
+    fillna : :obj:`bool`, optional
+        By default, if a Pitch class does not appear in a PCV, its value will be 0. Pass False if
+        you want NA instead.
+    additional_group_cols : (:obj:`list` of) :obj:`str`
+        If you would like to maintain some information from other columns of ``notes`` in additional index levels,
+        pass their names.
+    ensure_columns : :obj:`Iterable`, optional
+        By default, pitch classes that don't appear don't get a column. Pass a value if you want to
+        ensure the presence of particular columns, even if empty. For example, if ``pitch_class_format='pc'``
+        you could pass ``ensure_columns=range(12).
+
+
+    Returns
+    -------
+
+    """
+    if pitch_class_format in ("tpc", "name"):
+        pitch_class_grouper = notes.tpc.values
+    elif pitch_class_format == "pc":
+        pitch_class_grouper = (notes.midi % 12).values
+    elif pitch_class_format == "midi":
+        pitch_class_grouper = notes.midi.values
+    else:
+        logger.warning("pitch_class_format needs to be one of 'tpc', 'name', 'pc', 'midi', not "+ str(pitch_class_format))
+        return pd.DataFrame()
+    idx = notes.index
+    if isinstance(idx, pd.MultiIndex):
+        idx = idx.get_level_values(0)
+    grouper = [idx]
+    group_levels = 1
+    if additional_group_cols is not None:
+        if isinstance(additional_group_cols, str):
+            additional_group_cols = [additional_group_cols]
+        for col in additional_group_cols:
+            grouper.append(notes[col].values)
+            group_levels += 1
+    grouper.append(pitch_class_grouper)
+    pcvs = notes.groupby(grouper, dropna=False).duration_qb.sum()
+    # if long:
+    #     if normalize:
+    #         pcvs = pcvs.groupby(level=list(range(group_levels))).apply(lambda S: S / S.sum())
+    #     pcvs = pcvs.reset_index(level=-1)
+    #     if pitch_class_format == "name":
+    #         pcvs.loc[:, "tpc"] = transform(pcvs.tpc, fifths2name)
+    # else:
+    pcvs = pcvs.unstack()
+    if pitch_class_format == "name":
+        pcvs.columns = fifths2name(pcvs.columns)
+    if normalize:
+        pcvs = pcvs.div(pcvs.sum(axis=1), axis=0)
+    if ensure_columns is not None:
+        missing = [c for c in ensure_columns if c not in pcvs.columns]
+        if len(missing) > 0:
+            new_cols = pd.DataFrame(index=pcvs.index, columns=missing)
+            pcvs = pd.concat([pcvs, new_cols], axis=1).sort_index(axis=1)
+    if long:
+        pcvs = pcvs.stack()
+    if fillna:
+        return pcvs.fillna(0.)
+    return pcvs
 
 
 def resolve_all_relative_numerals(at, additional_columns=None, inplace=False):
@@ -605,6 +827,173 @@ def resolve_all_relative_numerals(at, additional_columns=None, inplace=False):
         return at
 
 
+@function_logger
+def segment_by_adjacency_groups(df, cols, na_values='group', group_keys=False):
+    """ Drop exact adjacent repetitions within one or a combination of several feature columns and adapt the
+    IntervalIndex and the column 'duration_qb' accordingly.
+    Uses: :py:func:`adjacency_groups`
+
+    Parameters
+    ----------
+    df : :obj:`pandas.DataFrame`
+        DataFrame to be reduced, expected to contain the column ``duration_qb``. In order to use the result as a
+        segmentation, it should have a :obj:`pandas.IntervalIndex`.
+    cols : :obj:`list`
+        Feature columns which exact, adjacent repetitions should be grouped to a segment, keeping only the first row.
+    na_values : (:obj:`list` of) :obj:`str` or :obj:`Any`, optional
+        | Either pass a list of equal length as ``cols`` or a single value that is passed to :py:func:`adjacency_groups`
+        | for each. Not dealing with NA values will lead to wrongly grouped segments. The default option is the safest.
+        | 'group' creates individual groups for NA values
+        | 'backfill' or 'bfill' groups NA values with the subsequent group
+        | 'pad', 'ffill' groups NA values with the preceding group
+        | Any other value works like 'group', with the difference that the created groups will be named with this value.
+    group_keys : :obj:`bool`, optional
+        By default, the grouped values will be returned as an appended MultiIndex, differentiation groups via ascending
+        integers. If you want to duplicate the columns' value, e.g. to account for a custom filling value for
+        ``na_values``, pass True. Beware that this most often results in non-unique index levels.
+
+    Returns
+    -------
+    :obj:`pandas.DataFrame`
+        Reduced DataFrame with updated 'duration_qb' column and :obj:`pandas.IntervalIndex` on the first level
+        (if present).
+
+    """
+    if 'duration_qb' not in df.columns:
+        logger.error("DataFrame is missing the column 'duration_qb'")
+    if isinstance(cols, str):
+        cols = [cols]
+    N = len(cols)
+    if not isinstance(na_values, list):
+        na_values = [na_values] * N
+    else:
+        while len(na_values) < N:
+            na_values.append('group')
+    groups, names = zip(
+        *(adjacency_groups(c, na_values=na) for (_, c), na in zip(df[cols].iteritems(), na_values)))
+
+    def sum_durations(df):
+        if len(df) == 1:
+            return df
+        idx = df.index
+        first_loc = idx[0]
+        row = df.iloc[[0]]
+        # if isinstance(ix, pd.Interval) or (isinstance(ix, tuple) and isinstance(ix[-1], pd.Interval)):
+        if isinstance(idx, pd.IntervalIndex):
+            start = min(idx.left)
+            end = max(idx.right)
+            iv = pd.Interval(start, end, closed=idx.closed)
+            row.index = pd.IntervalIndex([iv])
+            row.loc[iv, 'duration_qb'] = iv.length
+        else:
+            new_duration = df.duration_qb.sum()
+            row.loc[first_loc, 'duration_qb'] = new_duration
+        return row
+
+    grouped = df.groupby(list(groups), dropna=False).apply(sum_durations)
+    if any(gr.isna().any() for gr in groups):
+        logger.warning(
+            f"na_values={na_values} did not take care of all NA values in {cols}. This very probably leads to wrongly grouped rows. If in doubt, use 'group'.")
+    if isinstance(grouped.index, pd.MultiIndex):
+        grouped.index = grouped.index.reorder_levels([-1] + list(range(N)))
+    else:
+        logger.debug("Groupby-apply did not generate MultiIndex.")
+        grouped = grouped.set_index(list(groups), append=True)
+    if group_keys:
+        no_tuples = lambda coll: not any(isinstance(e, tuple) for e in coll)
+        new_levels = [index_level.map(d) if no_tuples(d.values()) else index_level for
+                      index_level, d in zip(grouped.index.levels[1:], names)]
+        grouped.index = grouped.index.set_levels(new_levels, level=list(range(1, N + 1)),
+                                                 verify_integrity=False)
+    grouped.index.rename('segment', level=0, inplace=True)
+    return grouped
+
+
+def segment_by_interval_index(df, idx, truncate=True):
+    """ Segment a DataFrame into chunks based on a given IntervalIndex.
+
+    Parameters
+    ----------
+    df : :obj:`pandas.DataFrame`
+        DataFrame that has a :obj:`pandas.IntervalIndex` to allow for its segmentation.
+    idx : :obj:`pandas.IntervalIndex` or :obj:`pandas.MultiIndex`
+        Intervals by which to segment ``df``. The index will be prepended to differentiate between segments. If ``idx``
+        is a :obj:`pandas.MultiIndex`, the first level is expected to be a :obj:`pandas.IntervalIndex`.
+    truncate : :obj:`bool`, optional
+        By default, the intervals of the segmented DataFrame will be cut off at segment boundaries and the event's
+        'duration_qb' will be adapted accordingly. Pass False to prevent that and duplicate overlapping events without
+        adapting their Intervals and 'duration_qb'.
+
+    Returns
+    -------
+    :obj:`pandas.DataFrame`
+        A copy of ``df`` where the index levels ``idx`` have been prepended and only rows of ``df`` with overlapping
+        intervals are included.
+
+    """
+    iv_idx = idx.copy()
+    if isinstance(iv_idx, pd.MultiIndex):
+        iv_idx = iv_idx.get_level_values(0)
+    assert isinstance(iv_idx, pd.IntervalIndex), f"idx needs to IntervalIndex, not {type(iv_idx)}"
+    chunks = []
+    for iv in iv_idx:
+        overlapping = df.index.overlaps(iv)
+        chunk = df[overlapping].copy()
+        if truncate:
+            start, end = iv.left, iv.right
+            chunk_index = chunk.index
+            left_overlap = chunk_index.left < start
+            right_overlap = chunk_index.right > end
+            if left_overlap.sum() > 0 or right_overlap.sum() > 0:
+                chunk.index = chunk_index.map(lambda i: interval_overlap(i, iv))
+                chunk.loc[:, 'duration_qb'] = chunk.index.length
+        chunks.append(chunk)
+    return pd.concat(chunks, keys=idx, levels=idx.levels)
+
+
+def slice_df(df, quarters_per_slice=None):
+    """ Returns a sliced version of the DataFrame. Slices appear in the IntervalIndex and the contained event's
+    durations within the slice are shown in the column 'duration_qb'.
+    Uses:
+
+    Parameters
+    ----------
+    df : :obj:`pandas.DataFrame`
+        The DataFrame is expected to come with an IntervalIndex and contain the columns 'quarterbeats' and 'duration_qb'.
+        Those can be obtained through ``Parse.get_lists(interval_index=True)`` or
+        ``Parse.iter_transformed(interval_index=True)``.
+    quarters_per_slice : :obj:`float`, optional
+        By default, the slices have variable size, from onset to onset. If you pass a value, the slices will
+        have that constant size, measured in quarter notes. For example, pass 1.0 for all slices to have size 1 quarter.
+
+    Returns
+    -------
+    :obj:`pandas.DataFrame`
+
+    """
+    assert isinstance(df.index, pd.IntervalIndex), f"DataFrame is expected to have an IntervalIndex, not {type(df.index)}."
+    end = max(df.index.right)
+    if quarters_per_slice is None:
+        lefts = sorted(set(df.index.left))
+        rights = lefts[1:] + [end]
+    else:
+        start = min(df.index.left)
+        lefts = np.arange(start, end, quarters_per_slice)
+        rights = np.arange(start + quarters_per_slice, end + quarters_per_slice, quarters_per_slice)
+    slices = []
+    for i, j in zip(lefts, rights):
+        iv = pd.Interval(i, j, closed='left')
+        overlapping = df.index.overlaps(iv)
+        overlapping_elements = df[overlapping].copy()
+        overlapping_elements.loc[:, 'duration_qb'] = [interval_overlap_size(ix, iv) for ix in
+                                                      overlapping_elements.index]
+        N = overlapping.sum()
+        new_index = pd.IntervalIndex([iv] * N, name='slice')
+        overlapping_elements.index = new_index
+        slices.append(overlapping_elements)
+    return pd.concat(slices)
+
+
 @function_logger
 def transform_multiple(df, func, level=-1, **kwargs):
     """ Applying transformation(s) separately to concatenated pieces that can be differentiated by index level(s).
@@ -648,10 +1037,6 @@ def transform_multiple(df, func, level=-1, **kwargs):
         logger.info(f"Index had too few levels ({list(df.index.names)}) to group by parameter level={levels}. Applied"
                     f"{func} to the entire DataFrame instead.")
         res = func(df, **kwargs)
-
-    # post-processing
-    if func == add_localkey_change_column:
-        res.key_segment = res.key_segment.cumsum()
     return res
 
 
@@ -678,6 +1063,44 @@ def transform_annotations(at, groupby_features=None, resolve_relative=False):
     return at
 
 
+def transpose_notes_to_localkey(notes):
+    """ Transpose the columns 'tpc' and 'midi' such that they reflect the local key as if it was C major/minor. This
+    operation is typically required for creating pitch class profiles.
+    Uses: :py:func:`transform`, :py:func:`name2fifths`, :py:func:`roman_numeral2fifths`
+
+    Parameters
+    ----------
+    notes : :obj:`pandas.DataFrame`
+        DataFrame that has at least the columns ['globalkey', 'localkey', 'tpc', 'midi'].
+
+    Returns
+    -------
+    :obj:`pandas.DataFrame`
+         A copy of ``notes`` where the columns 'tpc' and 'midi' are shifted in such a way that tpc=0 and midi=60
+         match the local tonic (e.g. for the local key A major/minor, each pitch A will have tpc=0 and midi % 12 = 0).
+    """
+    notes = notes.copy()
+    new_cols = []
+    if 'globalkey_is_minor' not in notes.columns:
+        gkim = notes.globalkey.str.islower().rename('globalkey_is_minor')
+        new_cols.append(gkim)
+    if 'localkey_is_minor' not in notes.columns:
+        lkim = notes.localkey.str.islower().rename('localkey_is_minor')
+        new_cols.append(lkim)
+    if len(new_cols) > 0:
+        notes = pd.concat([notes] + new_cols, axis=1)
+    transpose_by = transform(notes.globalkey, name2fifths) + transform(notes, roman_numeral2fifths, ['localkey', 'globalkey_is_minor'])
+    notes.tpc -= transpose_by
+    midi_transposition = transform(transpose_by, fifths2pc)
+    # For transpositions up to a diminished fifth, move pitches up,
+    # for larger intervals, move pitches down.
+    midi_transposition.where(midi_transposition <= 6, midi_transposition % -12, inplace=True)
+    notes.midi -= midi_transposition
+    notes = pd.concat([notes, transpose_by.rename('local_tonic')], axis=1)
+    notes.local_tonic = transform(notes, fifths2name, dict(fifths='local_tonic', minor='localkey_is_minor'))
+    return notes
+
+@function_logger
 def _treat_level_parameter(level, nlevels):
     """ Given a number of index levels, turn an int such as -1 into a list of all levels except the last.
 
@@ -693,15 +1116,19 @@ def _treat_level_parameter(level, nlevels):
     -------
 
     """
-    if isinstance(level, int):
+    if not isinstance(level, int):
+        return level
+    if level == 0:
+        level = nlevels
+    if level < 0:
+        level += nlevels
         if level < 0:
-            level += nlevels
-            if level < 0:
-                return []
-        elif level > nlevels:
-            level = nlevels
-        levels = list(range(level))
-    else:
-        levels = level
+            logger.warning(f"Cannot group by the {level} first levels. nlevels={nlevels}")
+            return []
+    elif level > nlevels:
+        logger.warning(f"Supposed to group by {level} first levels but nlevels={nlevels}")
+        return []
+    levels = list(range(level))
     return levels
 
+
